#!/usr/bin/perl -w
#
#	Simple CGI to pull from a MySQL database and display 
#	data in pretty tables.
#
#	06.19.04 Paul Venezia   pvenezia@jpj.net
#
#
#

use DBI;
use strict;

my $cgiurl = "/cgi-bin";	# CGI path relative to this server
my $script = "rdstats.cgi";	# Name of this script
my $host = `hostname`;

my ($query2,$sort,$location);

my $config_file = "/etc/mail/relaydelay.conf";


#################################################################
# Our global settings that may be overridden from the config file
#################################################################
         
# If you do/don't want to see debugging messages printed to stdout,
#   then set this appropriately.
my $verbose = 1;

# Database connection params
my $database_type = 'mysql';
my $database_name = 'relaydelay';
my $database_host = 'localhost';
my $database_port = 3306;
my $database_user = 'rduser';
my $database_pass = 'rdpass';

# Global vars that should probably not be in the external config file
my $global_dbh;
my $config_loaded;


#######################################################################
# Database functions
#######################################################################

sub db_connect($) {
  my $verbose = shift;

  return $global_dbh if (defined $global_dbh);

  my $dsn = "DBI:$database_type:database=$database_name:host=$database_host:port=$database_port";
  #print "DBI Connecting to $dsn\n" if $verbose;

  # Note: We do all manual error checking for db errors
  my $dbh = DBI->connect($dsn, $database_user, $database_pass,
                         { PrintError => 0, RaiseError => $verbose });

  $global_dbh = $dbh;
  return $global_dbh;
}

sub db_disconnect {
  $global_dbh->disconnect() if (defined $global_dbh);
  $global_dbh = undef;
  return 0;
}


sub load_config() {

  # make sure the config is only loaded once per instance
  return if ($config_loaded);

  #print "Loading Config File: $config_file\n";

  # Read and setup our configuration parameters from the config file
  my($msg);
  my($errn) = stat($config_file) ? 0 : 0+$!;
 # if ($errn == ENOENT) { $msg = "does not exist" }
  if ($errn)        { $msg = "inaccessible: $!" }
  elsif (! -f _)       { $msg = "not a regular file" }
  elsif (! -r _)       { $msg = "not readable" }
  if (defined $msg) { die "Config file $config_file $msg" }

  open INFILE, "<$config_file";
  while (<INFILE>) {
    my $tstr = $_;
    if ($tstr =~ /\A\s*(\$database_\w+)\s*=/) {
      eval $tstr;
      if ($@ ne '') { die "Error in config file $config_file: $@" }
    }
  }

  $config_loaded = 1;
}

# Securely read form data.
sub readform{

        my %fd                  = ();
	my $value		= "";
        my $name		= "";
        my $formdata            = $_[0];

        $formdata =~ s/\s+$//;

        foreach (split(/&/, $formdata)){
                ($name, $value)=split(/=/, $_);
                $name=~s/\+//g;
#               $name=~s/%([0-9|A-F]{2})/pack(C,hex($1))/eg;
                $name=~s/\W//eg;
                $value=~s/\+/ /g;
                $value=~ s/([;<>\*\|`&\$!#\(\)\[\]\{\}:'"])/\\$1/g;
                $value=~ s/([<>`&\$!#:'"])/\\$1/g;
                #$value=~s/%([0-9|A-F]{2})/pack(C,hex($1))/eg;
                if ($fd{$name}){
                        $fd{$name} = "$fd{$name},$value";
                }else{
                        $fd{$name} = $value;
                }
        }
        return %fd;
}

sub html_header{

 #       my $title = $_[0];
#        my $tbcol = $_[1];
#        my $opt   = $_[2];

        #print "Content-type:text/html\n\n";
        print "<HTML><HEAD><TITLE>RelayDelay Statstics</TITLE>\n";
        print "<meta Http-Equiv=Cache-Control Content=no-cache>\n";
        print "<meta Http-Equiv=Pragma Content=no-cache>\n";
        print "<meta Http-Equiv=Expires Content=now>\n";
	print "<LINK href=\"/relaydelay/style.css\" type=text/css rel=StyleSheet>\n";
	print "</HEAD>";
	print "<BODY>\n";
	print "<CENTER><H1>RelayDelay Statistics</H1></CENTER>\n";


}

sub error_handler() {
	&html_header;
	print "<CENTER><h3>Error in request</H3></CENTER>";
	&html_footer;
	exit;
}

sub html_footer () {
        print "</TD><tr></table><br><br>";
        print "<br><font size=1>Generated by rdstats.pl on " . localtime() . "</font><br>";
        print "</BODY></HTML>";
}


#my %data = &readform($ENV{QUERY_STRING});


  # load config file before we start
  load_config();
# Gimme database
	my $dbh = db_connect(1);

# Gimme sorting prefs
#if ($data{sort}) {
#	$sort = $data{sort};
#	$sort = "StnFreq ASC" if ($sort =~ /StnFreq/i);
#	$sort = "StnOnAir DESC" if ($sort =~ /StnOnAir/i);
#}else{
#	$sort = "StnMarket ASC";
#}

# Gimme data

  my $blrows = $dbh->do("SELECT id from relayreport where blocked_count >= 1 and passed_count = 0");
  my $parows = $dbh->do("SELECT id from relayreport where blocked_count >= 1 and passed_count >= 1");
  1 while $parows =~ s/(.*\d)(\d\d\d)/$1,$2/;
  1 while $blrows =~ s/(.*\d)(\d\d\d)/$1,$2/;

	my $toppassed = $dbh->selectall_arrayref("select sum(passed_count) as num, substring_index(relay_ip, '.', 3) as net, mail_from, rcpt_to from relaytofrom where passed_count > 0 group by substring_index(relay_ip, '.', 3) order by num desc limit 50", { Columns => {} });

	# look at most prolific blocked email counts by subnet
	my $topblocked = $dbh->selectall_arrayref("select sum(blocked_count) as num, substring_index(relay_ip, '.', 3) as net, mail_from, rcpt_to from relaytofrom where blocked_count > 0 AND passed_count = 0 group by substring_index(relay_ip, '.', 3) order by num desc limit 50", { Columns => {} });
	$dbh->disconnect;
	my ($cellcolor);

	&html_header;

# Actually do some output
        print "<CENTER><table border=0 cellspacing=1 bgcolor=#000000 cellpadding=8><tr><Th COLSPAN=2 BGCOLOR=#88AACC><font size=5>General Statistics for $host</font><BR><font size=1>Historical Data</FONT></th><tr>";   
        print "<TD ALIGN=LEFT bgcolor=\#9999FF>Blocked Without Retry</TD><TD ALIGN=RIGHT bgcolor=\#DCDCEE>$blrows</TD><TR>";
        print "<TD ALIGN=LEFT bgcolor=\#9999FF>Passed Following Retry</TD><TD ALIGN=RIGHT bgcolor=\#DCDCEE>$parows</TD>";
	print "</TABLE><BR>\n";

	print "<TABLE BORDER=0 CELLSPACING=1 CELLPADDING=8 BGCOLOR=#000000 WIDTH=20% ALIGN=center><TR BGCOLOR=#88AACC><TH align=center>Top 50 Passed Messages</TH></TR></TABLE><BR>\n";

	print "<TABLE BORDER=0 CELLSPACING=1 CELLPADDING=8 BGCOLOR=#8888AA WIDTH=90% ALIGN=CENTER>\n";
	print "<TR BGCOLOR=#A0C0E0><TH WIDTH=40>Count</TH><TH>Net</TH><TH>Sender</TH><TH>Recipient</TH></TR>\n";
	#print "<TR BGCOLOR=#A0C0E0><TH WIDTH=40><A HREF=\"$cgiurl/$script?state=$data{state}&sort=Count\">Count</TH><TH><A HREF=\"$cgiurl/$script?state=$data{state}&sort=Net\">Net</TH><TH><A HREF=\"$cgiurl/$script?state=$data{state}&sort=Recipient\">Recipient</TH><TH><A HREF=\"$cgiurl/$script?state=$data{state}&sort=Sender\">Sender</TH></TR>\n";

# Make it purty
for my $thisline (@$toppassed){
	if ($cellcolor =~ /#E0E0E0/) { 
		$cellcolor = "#C0C0CC";
	}else{
		$cellcolor = "#E0E0E0";
	}

	if ($thisline->{num} >= 1000 ) {
                $cellcolor = "#33dd00";
	}

# Gimme info
	$thisline->{mail_from} =~ s/(?:<|>)//g;
	$thisline->{rcpt_to} =~ s/(?:<|>)//g;
	$thisline->{net} = "<i>Whitelisted</i>" unless $thisline->{net};
	$thisline->{rcpt_to} = "<i>Whitelisted</i>" unless $thisline->{rcpt_to};
	$thisline->{mail_from} = "<i>Empty</i>" unless $thisline->{mail_from};
  	1 while $thisline->{num} =~ s/(.*\d)(\d\d\d)/$1,$2/;
	print "<TD BGCOLOR=$cellcolor>$thisline->{num}</TD><TD BGCOLOR=$cellcolor>$thisline->{net}</TD><TD BGCOLOR=$cellcolor>$thisline->{mail_from}</TD><TD BGCOLOR=$cellcolor>$thisline->{rcpt_to}</TD><TR>";
}
	
	print "</TABLE><BR>\n";
	
	print "<TABLE BORDER=0 CELLSPACING=1 CELLPADDING=8 BGCOLOR=#000000 WIDTH=20% ALIGN=center><TR BGCOLOR=#88AACC><TH align=center>Top 50 Blocked Messages</TH></TR></TABLE><BR>\n";

	print "<TABLE BORDER=0 CELLSPACING=1 CELLPADDING=8 BGCOLOR=#8888AA WIDTH=90% ALIGN=CENTER>\n";
	print "<TR BGCOLOR=#A0C0E0><TH WIDTH=40>Count</TH><TH>Net</TH><TH>Sender</TH><TH>Recipient</TH></TR>\n";
	#print "<TR BGCOLOR=#A0C0E0><TH WIDTH=40><A HREF=\"$cgiurl/$script?state=$data{state}&sort=Count\">Count</TH><TH><A HREF=\"$cgiurl/$script?state=$data{state}&sort=Net\">Net</TH><TH><A HREF=\"$cgiurl/$script?state=$data{state}&sort=Recipient\">Recipient</TH><TH><A HREF=\"$cgiurl/$script?state=$data{state}&sort=Sender\">Sender</TH></TR>\n";

for my $thisline (@$topblocked){
        if ($cellcolor =~ /#E0E0E0/) {
                $cellcolor = "#C0C0CC";
        }else{
                $cellcolor = "#E0E0E0";
        }

        if ($thisline->{num} >= 1000 ) {
		$cellcolor = "#d9787a";
        }

# Gimme info
	$thisline->{mail_from} =~ s/(?:<|>)//g;
	$thisline->{rcpt_to} =~ s/(?:<|>)//g;
	$thisline->{net} = "<i>Blacklisted</i>" unless $thisline->{net};
	$thisline->{rcpt_to} = "<i>Blacklisted</i>" unless $thisline->{rcpt_to};
	$thisline->{mail_from} = "<i>Empty</i>" unless $thisline->{mail_from};
  	1 while $thisline->{num} =~ s/(.*\d)(\d\d\d)/$1,$2/;
        print "<TD BGCOLOR=$cellcolor>$thisline->{num}</TD><TD BGCOLOR=$cellcolor>$thisline->{net}</TD><TD BGCOLOR=$cellcolor>$thisline->{mail_from}</TD><TD BGCOLOR=$cellcolor>$thisline->{rcpt_to}</TD><TR>";
}
	print "</TABLE><BR>\n";
	&html_footer;
print "</BODY></HTML>";

# Gimme EOF
